@@ -3,27 +3,30 @@ import sys
 
 from django.conf import settings
 from django.forms.forms import BoundField
+from django.template import Context
 from django.template.loader import get_template
 from django.utils.html import conditional_escape
+from django.utils.functional import SimpleLazyObject
 
 
-# Global field template, default template used for rendering a field. This way we avoid 
+# Global field template, default template used for rendering a field. This way we avoid
 # loading the template every time render_field is called without a template
 TEMPLATE_PACK = getattr(settings, 'CRISPY_TEMPLATE_PACK', 'bootstrap')
-default_field_template = get_template("%s/field.html" % TEMPLATE_PACK)
+default_field_template = SimpleLazyObject(lambda: get_template("%s/field.html" % TEMPLATE_PACK))
+
 
 def render_field(field, form, form_style, context, template=None, labelclass=None, layout_object=None, attrs=None):
     """
     Renders a django-crispy-forms field
-    
+
     :param field: Can be a string or a Layout object like `Row`. If it's a layout
         object, we call its render method, otherwise we instantiate a BoundField
-        and render it using default template 'uni_form/field.html'
+        and render it using default template 'CRISPY_TEMPLATE_PACK/field.html'
         The field is added to a list that the form holds called `rendered_fields`
         to avoid double rendering fields.
 
     :param form: The form/formset to which that field belongs to.
-    
+
     :param form_style: A way to pass style name to the CSS framework used.
 
     :template: Template used for rendering the field.
@@ -43,10 +46,10 @@ def render_field(field, form, form_style, context, template=None, labelclass=Non
             if isinstance(field, unicode):
                 field = str(field)
             # If `field` is not unicode then we turn it into a unicode string, otherwise doing
-            # str(field) would give no error and the field would not be resolved, causing confusion 
+            # str(field) would give no error and the field would not be resolved, causing confusion
             else:
                 field = str(unicode(field))
-                
+
         except (UnicodeEncodeError, UnicodeDecodeError):
             raise Exception("Field '%s' is using forbidden unicode characters" % field)
 
@@ -54,14 +57,34 @@ def render_field(field, form, form_style, context, template=None, labelclass=Non
         # Injecting HTML attributes into field's widget, Django handles rendering these
         field_instance = form.fields[field]
         if attrs is not None:
-            field_instance.widget.attrs.update(attrs)
+            widgets = getattr(field_instance.widget, 'widgets', [field_instance.widget,])
+
+            # We use attrs as a dictionary later, so here we make a copy
+            list_attrs = attrs
+            if isinstance(attrs, dict):
+                list_attrs = [attrs] * len(widgets)
+
+            for index, (widget, attr) in enumerate(zip(widgets, list_attrs)):
+                if hasattr(field_instance.widget, 'widgets'):
+                    if 'type' in attr and attr['type'] == "hidden":
+                        field_instance.widget.widgets[index].is_hidden = True
+                        field_instance.widget.widgets[index] = field_instance.hidden_widget()
+
+                    field_instance.widget.widgets[index].attrs.update(attr)
+                else:
+                    if 'type' in attr and attr['type'] == "hidden":
+                        field_instance.widget.is_hidden = True
+                        field_instance.widget = field_instance.hidden_widget()
+
+                    field_instance.widget.attrs.update(attr)
+
     except KeyError:
         if not FAIL_SILENTLY:
             raise Exception("Could not resolve form field '%s'." % field)
         else:
             field_instance = None
             logging.warning("Could not resolve form field '%s'." % field, exc_info=sys.exc_info())
-            
+
     if not field in form.rendered_fields:
         form.rendered_fields.add(field)
     else:
@@ -84,7 +107,11 @@ def render_field(field, form, form_style, context, template=None, labelclass=Non
         if layout_object is not None:
             layout_object.bound_fields.append(bound_field)
 
-        context.update({'field': bound_field, 'labelclass': labelclass, 'flat_attrs': flatatt(attrs or {})})
+        context.update({
+            'field': bound_field,
+            'labelclass': labelclass,
+            'flat_attrs': flatatt(attrs if isinstance(attrs, dict) else {}),
+        })
         html = template.render(context)
 
     return html
@@ -92,10 +119,34 @@ def render_field(field, form, form_style, context, template=None, labelclass=Non
 
 def flatatt(attrs):
     """
-    Taken from django.core.utils 
+    Taken from django.core.utils
     Convert a dictionary of attributes to a single string.
     The returned string will contain a leading space followed by key="value",
     XML-style pairs.  It is assumed that the keys do not need to be XML-escaped.
     If the passed dictionary is empty, then return an empty string.
     """
     return u''.join([u' %s="%s"' % (k.replace('_', '-'), conditional_escape(v)) for k, v in attrs.items()])
+
+
+def render_crispy_form(form, helper=None, context=None):
+    """
+    Renders a form and returns its HTML output.
+
+    This function wraps the template logic in a function easy to use in a Django view.
+    """
+    from crispy_forms.templatetags.crispy_forms_tags import CrispyFormNode
+
+    if helper is not None:
+        node = CrispyFormNode('form', 'helper')
+    else:
+        node = CrispyFormNode('form', None)
+
+    node_context = Context({
+        'form': form,
+        'helper': helper
+    })
+
+    if context is not None:
+        node_context.update(context)
+
+    return node.render(node_context)
